@@ -26,9 +26,7 @@
 
 __author__ = "Daniel Scheffler"
 
-import os
 import warnings
-from tempfile import TemporaryDirectory
 import json
 from typing import Union
 
@@ -100,59 +98,68 @@ def raster2polygon(array: np.ndarray,
     srs = osr.SpatialReference()
     srs.ImportFromWkt(prj)
 
-    with TemporaryDirectory() as td:
-        drv = ogr.GetDriverByName("GeoJSON")
-        path_geojson = os.path.join(td, "polygonize_result.geojson")
-        dst_ds = drv.CreateDataSource(path_geojson)
-        dst_layer = dst_ds.CreateLayer('polygonize_result', srs=srs)
-        dst_layer.CreateField(ogr.FieldDefn("DN", 4))
+    drv = ogr.GetDriverByName("GeoJSON")
+    path_geojson = "/vsimem/polygonize_result.geojson"
+    dst_ds = drv.CreateDataSource(path_geojson)
+    dst_layer = dst_ds.CreateLayer('polygonize_result', srs=srs)
+    dst_layer.CreateField(ogr.FieldDefn("DN", 4))
 
-        # set callback
-        callback = \
-            ProgressBar(prefix='Polygonize progress    ',
-                        suffix='Complete',
-                        barLength=50,
-                        timeout=timeout,
-                        use_as_callback=True) \
-            if progress and not q else Timer(timeout, use_as_callback=True) if timeout else None
+    # set callback
+    callback = \
+        ProgressBar(prefix='Polygonize progress    ',
+                    suffix='Complete',
+                    barLength=50,
+                    timeout=timeout,
+                    use_as_callback=True) \
+        if progress and not q else Timer(timeout, use_as_callback=True) if timeout else None
 
-        # run the algorithm
-        status = gdal.Polygonize(src_band,
-                                 src_band,  # .GetMaskBand(),
-                                 dst_layer,
-                                 0,
-                                 ["8CONNECTED=8"] if exact else [],
-                                 callback=callback)
+    # run the algorithm
+    status = gdal.Polygonize(src_band,
+                             src_band,  # .GetMaskBand(),
+                             dst_layer,
+                             0,
+                             ["8CONNECTED=8"] if exact else [],
+                             callback=callback)
 
-        del dst_layer, dst_ds
+    del dst_layer, dst_ds
 
-        # handle exit status other than 0 (fail)
-        if status != 0:
-            errMsg = gdal.GetLastErrorMsg()
+    # handle exit status other than 0 (fail)
+    if status != 0:
+        errMsg = gdal.GetLastErrorMsg()
 
-            # Catch the KeyboardInterrupt raised in case of a timeout within the callback.
-            # It seems like there is no other way to catch exceptions within callbacks.
-            if errMsg == 'User terminated':
-                raise TimeoutError('raster2polygon timed out!')
+        # Catch the KeyboardInterrupt raised in case of a timeout within the callback.
+        # It seems like there is no other way to catch exceptions within callbacks.
+        if errMsg == 'User terminated':
+            raise TimeoutError('raster2polygon timed out!')
 
-            raise Exception(errMsg)
+        raise Exception(errMsg)
 
-        with open(path_geojson) as f:
-            gjs = json.load(f)
+    # read virtual GeoJSON file as text
+    f = gdal.VSIFOpenL(path_geojson, 'r')
+    gdal.VSIFSeekL(f, 0, 2)  # seek to end
+    size = gdal.VSIFTellL(f)
+    gdal.VSIFSeekL(f, 0, 0)  # seek to beginning
+    content_str = gdal.VSIFReadL(1, size, f)
+    gdal.VSIFCloseL(f)
+
+    gdal.Unlink(path_geojson)
+
+    # convert JSON string to dict
+    gjs = json.loads(content_str)
 
     gc = GeometryCollection([shape(f["geometry"]).buffer(0)
                              for f in gjs['features'] if f['properties']['DN'] == str(DN2extract)])
 
     # convert GeometryCollection into MultiPolygon
     polys = []
-    for i in gc:
+    for i in gc.geoms:
         if isinstance(i, Polygon):
             polys.append(i)
         else:
-            polys.extend(list(i))
+            polys.extend(list(i.geoms))
 
     # drop polygons with an area below npx
-    if min_npx:
+    if min_npx > 1:
         area_1px = gt[1] * abs(gt[5])
         area_min = min_npx * area_1px
 
